@@ -120,18 +120,18 @@ sub schema_type {
   return _guessed_right(object => $_[1]) if $_[0]->{additionalProperties};
   return _guessed_right(object => $_[1]) if $_[0]->{patternProperties};
   return _guessed_right(object => $_[1]) if $_[0]->{properties};
-  return _guessed_right(object => $_[1]) if $_[0]->{propertyNames};
+  return _guessed_right(object => $_[1]) if exists $_[0]->{propertyNames};
   return _guessed_right(object => $_[1]) if $_[0]->{required};
   return _guessed_right(object => $_[1]) if $_[0]->{dependencies};
   return _guessed_right(object => $_[1]) if defined $_[0]->{maxProperties} or defined $_[0]->{minProperties};
 
   # additionalItems is intentionally omitted - it requires 'items' to take effect
-  return _guessed_right(array  => $_[1]) if defined $_[0]->{items};
+  return _guessed_right(array  => $_[1]) if exists $_[0]->{items};
   return _guessed_right(array  => $_[1]) if $_[0]->{uniqueItems};
-  return _guessed_right(array  => $_[1]) if defined $_[0]->{contains};
-  return _guessed_right(array  => $_[1]) if defined $_[0]->{maxItems} or defined $_[0]->{minItems};
+  return _guessed_right(array  => $_[1]) if exists $_[0]->{contains};
+  return _guessed_right(array  => $_[1]) if exists $_[0]->{maxItems} or exists $_[0]->{minItems};
   return _guessed_right(string => $_[1]) if $_[0]->{pattern};
-  return _guessed_right(string => $_[1]) if defined $_[0]->{maxLength} or defined $_[0]->{minLength};
+  return _guessed_right(string => $_[1]) if exists $_[0]->{maxLength} or defined $_[0]->{minLength};
   return _guessed_right(number => $_[1]) if $_[0]->{multipleOf};
   return _guessed_right(number => $_[1])
     if defined $_[0]->{maximum}
@@ -176,7 +176,7 @@ sub _schema_extract {
       return undef;
     }
 
-    $data = $tied->schema if ref $data eq 'HASH' and $tied = tied %$data;
+    $data = $tied->schema while ref $data eq 'HASH' and $tied = tied %$data;
   }
 
   return $cb->($data, $pos) if $cb;
